@@ -156,7 +156,10 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         for line in out.splitlines():
             if line:
                 info = line.split()
-                lines.append((info[0].split(',')[0], info[1]))
+                try:
+                    lines.append((info[0].split(',')[0], info[1]))
+                except IndexError:
+                    pass
         return lines
 
     def _get_iscsi_sessions(self):
@@ -737,11 +740,11 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
                     data['failed_logins'])):
             # We have devices but we don't know the wwn yet
             if not wwn and found:
-                wwn = self._linuxscsi.get_sysfs_wwn(found)
-            # We have the wwn but not a multipath
-            if wwn and not mpath:
+                wwn = self._linuxscsi.get_sysfs_wwn(found, mpath)
+            if not mpath and found:
                 mpath = self._linuxscsi.find_sysfs_multipath_dm(found)
-                if not (mpath or wwn_added):
+                # We have the wwn but not a multipath
+                if wwn and not(mpath or wwn_added):
                     # Tell multipathd that this wwn is a multipath and hint
                     # multipathd to recheck all the devices we have just
                     # connected.  We only do this once, since for any new
@@ -778,6 +781,8 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         if not mpath:
             LOG.warning('No dm was created, connection to volume is probably '
                         'bad and will perform poorly.')
+        elif not wwn:
+            wwn = self._linuxscsi.get_sysfs_wwn(found, mpath)
         return self._get_connect_result(connection_properties, wwn, found,
                                         mpath)
 
