@@ -88,12 +88,20 @@ class ScaleIOConnector(base.BaseLinuxConnector):
             raise exception.BrickException(message=msg)
 
     @staticmethod
-    def _get_connector_password(config_group, failed_over):
+    def _get_password_token(connection_properties):
+        # In old connection format we had the password and token in properties
+        if 'serverPassword' in connection_properties:
+            return (connection_properties['serverPassword'],
+                    connection_properties['serverToken'])
+
+        # The new format reads password from file and doesn't have the token
         LOG.info("Get ScaleIO connector password from configuration file")
         try:
-            return priv_scaleio.get_connector_password(CONNECTOR_CONF_PATH,
-                                                       config_group,
-                                                       failed_over)
+            password = priv_scaleio.get_connector_password(
+                CONNECTOR_CONF_PATH,
+                connection_properties['config_group'],
+                connection_properties.get('failed_over', False))
+            return password, None
         except Exception as e:
             msg = _("Error getting ScaleIO connector password from "
                     "configuration file: %s") % e
@@ -320,10 +328,8 @@ class ScaleIOConnector(base.BaseLinuxConnector):
         self.server_ip = connection_properties['serverIP']
         self.server_port = connection_properties['serverPort']
         self.server_username = connection_properties['serverUsername']
-        self.server_password = self._get_connector_password(
-            connection_properties['config_group'],
-            connection_properties['failed_over'],
-        )
+        self.server_password, self.server_token = self._get_password_token(
+            connection_properties)
         self.iops_limit = connection_properties['iopsLimit']
         self.bandwidth_limit = connection_properties['bandwidthLimit']
         device_info = {'type': 'block',
