@@ -127,8 +127,20 @@ class LinuxSCSI(executor.Executor):
         LOG.debug('dev_info=%s', str(dev_info))
         return dev_info
 
-    def get_sysfs_wwn(self, device_names):
+    def get_sysfs_wwn(self, device_names, mpath=None):
         """Return the wwid from sysfs in any of devices in udev format."""
+        # If we have a multipath DM we know that it has found the WWN
+        if mpath:
+            # We have the WWN in /uuid even with friendly names, unline /name
+            try:
+                with open('/sys/block/%s/dm/uuid' % mpath) as f:
+                    # Contents are matph-WWN, so get the part we want
+                    wwid = f.read().strip()[6:]
+                    if wwid:  # Check should not be needed, but just in case
+                        return wwid
+            except Exception as exc:
+                LOG.warning('Failed to read the DM uuid: %s', exc)
+
         wwid = self.get_sysfs_wwid(device_names)
         glob_str = '/dev/disk/by-id/scsi-'
         wwn_paths = glob.glob(glob_str + '*')
@@ -136,16 +148,34 @@ class LinuxSCSI(executor.Executor):
         if wwid and glob_str + wwid in wwn_paths:
             return wwid
 
-        # If we have multiple designators follow the symlinks
+        # If we have multiple designators use symlinks to find out the wwn
+        device_names = set(device_names)
         for wwn_path in wwn_paths:
             try:
                 if os.path.islink(wwn_path) and os.stat(wwn_path):
                     path = os.path.realpath(wwn_path)
-                    if path.startswith('/dev/') and path[5:] in device_names:
-                        return wwn_path[len(glob_str):]
+                    if path.startswith('/dev/'):
+                        name = path[5:]
+                        # Symlink may point to the multipath dm if the attach
+                        # was too fast or we took long to check it. Check
+                        # devices belonging to the multipath DM.
+                        if name.startswith('dm-'):
+                            # Get the devices that belong to the DM
+                            slaves_path = '/sys/class/block/%s/slaves' % name
+                            dm_devs = os.listdir(slaves_path)
+                            # This is the right wwn_path if the devices we have
+                            # attached belong to the dm we followed
+                            if device_names.intersection(dm_devs):
+                                break
+
+                        # This is the right wwn_path if  devices we have
+                        elif name in device_names:
+                            break
             except OSError:
                 continue
-        return ''
+        else:
+            return ''
+        return wwn_path[len(glob_str):]
 
     def get_sysfs_wwid(self, device_names):
         """Return the wwid from sysfs in any of devices in udev format."""
@@ -240,16 +270,23 @@ class LinuxSCSI(executor.Executor):
         """
         # No used path happens on failed attachs, when we don't care about
         # individual flushes.
-        # When Nova used a multipath we don't need to do individual flushes.
-        if not path_used or was_multipath:
+        if not path_used:
             return False
 
-        # We need to flush the single path that was used.
-        # For encrypted volumes the symlink has been replaced, so realpath
-        # won't return device under /dev but under /dev/disk/...
         path = os.path.realpath(path)
         path_used = os.path.realpath(path_used)
-        return path_used == path or '/dev' != os.path.split(path_used)[0]
+
+        # Need to flush this device if we used this specific path.  We check
+        # this before checking if it's multipath in case we don't detect it
+        # being multipath correctly (as in bug #1897787).
+        if path_used == path:
+            return True
+
+        # We flush individual path if Nova didn't use a multipath and we
+        # replaced the symlink to a real device with a link to the decrypted
+        # DM.  We know we replaced it because it doesn't link to /dev/XYZ,
+        # instead it maps to /dev/mapped/crypt-XYZ
+        return not was_multipath and '/dev' != os.path.split(path_used)[0]
 
     def remove_connection(self, devices_names, is_multipath, force=False,
                           exc=None, path_used=None, was_multipath=False):
