@@ -14,7 +14,7 @@
 #    under the License.
 
 import binascii
-import mock
+from unittest import mock
 
 from castellan.common.objects import symmetric_key as key
 from os_brick.encryptors import luks
@@ -227,6 +227,7 @@ class LuksEncryptorTestCase(test_cryptsetup.CryptsetupEncryptorTestCase):
                       root_helper=self.root_helper, run_as_root=True,
                       check_exit_code=[0, 4], attempts=3),
             mock.call('cryptsetup', 'luksAddKey', self.dev_path,
+                      '--force-password',
                       process_input=''.join([fake_key_mangled,
                                              '\n', fake_key,
                                              '\n', fake_key]),
@@ -253,3 +254,62 @@ class LuksEncryptorTestCase(test_cryptsetup.CryptsetupEncryptorTestCase):
                       check_exit_code=True),
         ], any_order=False)
         self.assertEqual(9, mock_execute.call_count)
+
+
+class Luks2EncryptorTestCase(LuksEncryptorTestCase):
+    def _create(self):
+        return luks.Luks2Encryptor(root_helper=self.root_helper,
+                                   connection_info=self.connection_info,
+                                   keymgr=self.keymgr)
+
+    @mock.patch('os_brick.executor.Executor._execute')
+    def test__format_volume(self, mock_execute):
+        self.encryptor._format_volume("passphrase")
+
+        mock_execute.assert_has_calls([
+            mock.call('cryptsetup', '--batch-mode', 'luksFormat',
+                      '--type', 'luks2', '--key-file=-', self.dev_path,
+                      process_input='passphrase',
+                      root_helper=self.root_helper,
+                      run_as_root=True, check_exit_code=True, attempts=3),
+        ])
+
+    @mock.patch('os_brick.executor.Executor._execute')
+    def test_attach_volume_not_formatted(self, mock_execute):
+        fake_key = 'bc37c5eccebe403f9cc2d0dd20dac2bc'
+        self.encryptor._get_key = mock.MagicMock()
+        self.encryptor._get_key.return_value = (
+            test_cryptsetup.fake__get_key(None, fake_key))
+
+        mock_execute.side_effect = [
+            putils.ProcessExecutionError(exit_code=1),  # luksOpen
+            putils.ProcessExecutionError(exit_code=1),  # isLuks
+            mock.DEFAULT,  # luksFormat
+            mock.DEFAULT,  # luksOpen
+            mock.DEFAULT,  # ln
+        ]
+
+        self.encryptor.attach_volume(None)
+
+        mock_execute.assert_has_calls([
+            mock.call('cryptsetup', 'luksOpen', '--key-file=-', self.dev_path,
+                      self.dev_name, process_input=fake_key,
+                      root_helper=self.root_helper,
+                      run_as_root=True, check_exit_code=True),
+            mock.call('cryptsetup', 'isLuks', '--verbose', self.dev_path,
+                      root_helper=self.root_helper,
+                      run_as_root=True, check_exit_code=True),
+            mock.call('cryptsetup', '--batch-mode', 'luksFormat',
+                      '--type', 'luks2', '--key-file=-', self.dev_path,
+                      process_input=fake_key,
+                      root_helper=self.root_helper,
+                      run_as_root=True, check_exit_code=True, attempts=3),
+            mock.call('cryptsetup', 'luksOpen', '--key-file=-', self.dev_path,
+                      self.dev_name, process_input=fake_key,
+                      root_helper=self.root_helper,
+                      run_as_root=True, check_exit_code=True),
+            mock.call('ln', '--symbolic', '--force',
+                      '/dev/mapper/%s' % self.dev_name, self.symlink_path,
+                      root_helper=self.root_helper,
+                      run_as_root=True, check_exit_code=True),
+        ], any_order=False)
