@@ -860,6 +860,16 @@ loop0                                     0"""
                                     'id': '0',
                                     'lun': '0'})
 
+    @mock.patch('six.moves.builtins.open')
+    def test_get_sysfs_wwn_mpath(self, open_mock):
+        wwn = '3600d0230000000000e13955cc3757800'
+        cm_open = open_mock.return_value.__enter__.return_value
+        cm_open.read.return_value = 'mpath-' + wwn
+
+        res = self.linuxscsi.get_sysfs_wwn(mock.sentinel.device_names, 'dm-1')
+        open_mock.assert_called_once_with('/sys/block/dm-1/dm/uuid')
+        self.assertEqual(wwn, res)
+
     @mock.patch('glob.glob')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_sysfs_wwid')
     def test_get_sysfs_wwn_single_designator(self, get_wwid_mock, glob_mock):
@@ -871,25 +881,67 @@ loop0                                     0"""
         glob_mock.assert_called_once_with('/dev/disk/by-id/scsi-*')
         get_wwid_mock.assert_called_once_with(mock.sentinel.device_names)
 
+    @mock.patch('six.moves.builtins.open', side_effect=Exception)
+    @mock.patch('glob.glob')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_sysfs_wwid')
+    def test_get_sysfs_wwn_mpath_exc(self, get_wwid_mock, glob_mock,
+                                     open_mock):
+        glob_mock.return_value = ['/dev/disk/by-id/scsi-wwid1',
+                                  '/dev/disk/by-id/scsi-wwid2']
+        get_wwid_mock.return_value = 'wwid1'
+        res = self.linuxscsi.get_sysfs_wwn(mock.sentinel.device_names, 'dm-1')
+        open_mock.assert_called_once_with('/sys/block/dm-1/dm/uuid')
+        self.assertEqual('wwid1', res)
+        glob_mock.assert_called_once_with('/dev/disk/by-id/scsi-*')
+        get_wwid_mock.assert_called_once_with(mock.sentinel.device_names)
+
+    @mock.patch('os.listdir', return_value=['sda', 'sdd'])
     @mock.patch('os.path.realpath', side_effect=('/other/path',
+                                                 '/dev/dm-5',
                                                  '/dev/sda', '/dev/sdb'))
-    @mock.patch('os.path.islink', side_effect=(False, True, True, True, True))
-    @mock.patch('os.stat', side_effect=(False, True, True, True))
+    @mock.patch('os.path.islink', side_effect=(False,) + (True,) * 5)
+    @mock.patch('os.stat', side_effect=(False,) + (True,) * 4)
     @mock.patch('glob.glob')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_sysfs_wwid')
     def test_get_sysfs_wwn_multiple_designators(self, get_wwid_mock, glob_mock,
                                                 stat_mock, islink_mock,
-                                                realpath_mock):
+                                                realpath_mock, listdir_mock):
         glob_mock.return_value = ['/dev/disk/by-id/scsi-fail-link',
                                   '/dev/disk/by-id/scsi-fail-stat',
                                   '/dev/disk/by-id/scsi-non-dev',
+                                  '/dev/disk/by-id/scsi-another-dm',
                                   '/dev/disk/by-id/scsi-wwid1',
                                   '/dev/disk/by-id/scsi-wwid2']
+
         get_wwid_mock.return_value = 'pre-wwid'
         devices = ['sdb', 'sdc']
         res = self.linuxscsi.get_sysfs_wwn(devices)
         self.assertEqual('wwid2', res)
         glob_mock.assert_called_once_with('/dev/disk/by-id/scsi-*')
+        listdir_mock.assert_called_once_with('/sys/class/block/dm-5/slaves')
+        get_wwid_mock.assert_called_once_with(devices)
+
+    @mock.patch('os.listdir', side_effect=[['sda', 'sdb'], ['sdc', 'sdd']])
+    @mock.patch('os.path.realpath', side_effect=('/dev/sde',
+                                                 '/dev/dm-5',
+                                                 '/dev/dm-6'))
+    @mock.patch('os.path.islink', mock.Mock())
+    @mock.patch('os.stat', mock.Mock())
+    @mock.patch('glob.glob')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_sysfs_wwid', return_value='')
+    def test_get_sysfs_wwn_dm_link(self, get_wwid_mock, glob_mock,
+                                   realpath_mock, listdir_mock):
+        glob_mock.return_value = ['/dev/disk/by-id/scsi-wwid1',
+                                  '/dev/disk/by-id/scsi-another-dm',
+                                  '/dev/disk/by-id/scsi-our-dm']
+
+        devices = ['sdc', 'sdd']
+        res = self.linuxscsi.get_sysfs_wwn(devices)
+        self.assertEqual('our-dm', res)
+        glob_mock.assert_called_once_with('/dev/disk/by-id/scsi-*')
+        listdir_mock.assert_has_calls(
+            [mock.call('/sys/class/block/dm-5/slaves'),
+             mock.call('/sys/class/block/dm-6/slaves')])
         get_wwid_mock.assert_called_once_with(devices)
 
     @mock.patch('os.path.realpath', side_effect=('/dev/sda', '/dev/sdb'))
@@ -908,6 +960,16 @@ loop0                                     0"""
         glob_mock.assert_called_once_with('/dev/disk/by-id/scsi-*')
         get_wwid_mock.assert_called_once_with(devices)
 
+    @mock.patch('glob.glob', return_value=[])
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_sysfs_wwid')
+    def test_get_sysfs_wwn_no_links(self, get_wwid_mock, glob_mock):
+        get_wwid_mock.return_value = ''
+        devices = ['sdc']
+        res = self.linuxscsi.get_sysfs_wwn(devices)
+        self.assertEqual('', res)
+        glob_mock.assert_called_once_with('/dev/disk/by-id/scsi-*')
+        get_wwid_mock.assert_called_once_with(devices)
+
     @ddt.data({'wwn_type': 't10.', 'num_val': '1'},
               {'wwn_type': 'eui.', 'num_val': '2'},
               {'wwn_type': 'naa.', 'num_val': '3'})
@@ -1076,8 +1138,15 @@ loop0                                     0"""
     def test_get_dev_path_no_path(self, con_props, dev_info):
         self.assertEqual('', self.linuxscsi.get_dev_path(con_props, dev_info))
 
-    @ddt.data(('/dev/sda', '/dev/sda', True, False, None),
+    @ddt.data(('/dev/sda', '/dev/sda', False, True, None),
+              # This checks that we ignore the was_multipath parameter if it
+              # doesn't make sense (because the used path is the one we are
+              # asking about)
+              ('/dev/sda', '/dev/sda', True, True, None),
               ('/dev/sda', '', True, False, None),
+              # Check for encrypted volume
+              ('/dev/link_sda', '/dev/disk/by-path/pci-XYZ', False, True,
+               ('/dev/sda', '/dev/mapper/crypt-pci-XYZ')),
               ('/dev/link_sda', '/dev/link_sdb', False, False, ('/dev/sda',
                                                                 '/dev/sdb')),
               ('/dev/link_sda', '/dev/link2_sda', False, True, ('/dev/sda',
